@@ -27,7 +27,25 @@ def threeD_to_phase_const(T, theta, s=1):
     """Simply the inverse of phase_to_threeD_const"""
     return 1 / phase_to_threeD_const(T, theta, s)
 
-_min_percentile = 0.005
+def get_scales(calibration, shape=None):
+    if 'scale' in calibration:
+        scale = calibration['scale']
+    else:
+        if shape is None:
+            raise ValueError("Shape must be given if scale is not found in calibration")
+        # if no scale, make the output have a maximum size of 5
+        scale = np.max(shape) / 5
+
+    xscale = yscale = dscale = 1. / scale
+    if 'theta' in calibration:
+        xscale *= 1 / np.cos(calibration['theta'])
+        dscale *= phase_to_threeD_const(calibration['T'], calibration['theta'])
+    if 'phi' in calibration:  ## Phi stuff is not really tested 
+        dscale *= 1 / np.cos(np.abs(calibration['phi'] - calibration['gamma']))
+    return xscale, yscale, dscale
+
+
+# _min_percentile = 0.005
 def fp23d(signal, calibration, negative_theta=False):
     """Function for 3D reconstruction of a fringe pattern, if no calibration has been performed you can call the automated calibration fuction `fp23dpy.Calibration.calibrate` and use that as input calibration"""
     isMasked = np.ma.isMaskedArray(signal)
@@ -37,10 +55,10 @@ def fp23d(signal, calibration, negative_theta=False):
         roi = Roi.find_from_mask(signal.mask)
         signal = roi.apply(signal)
         mask = signal.mask
-        principal_point = roi.apply_to_points([calibration['principal_point']])[0] if 'principal_point' in calibration else np.array(signal.shape) / 2
+        principal_point = roi.apply_to_points([calibration['principal_point']])[0] if 'principal_point' in calibration else np.array(signal.shape[::-1]) / 2
     else:
-        mask = np.zeros(signal.shape)
-        principal_point = calibration['principal_point'] if 'principal_point' in calibration else np.array(signal.shape) / 2
+        mask = np.zeros(signal.shape, dtype=bool)
+        principal_point = calibration['principal_point'] if 'principal_point' in calibration else np.array(signal.shape[::-1]) / 2
 
     shape = signal.shape
 
@@ -49,34 +67,36 @@ def fp23d(signal, calibration, negative_theta=False):
     carrier = h.make_carrier(signal.shape[-2:], calibration['T'], calibration['gamma'], principal_point)
     phase = (1 - 2 * negative_theta) * (phase_with_carrier - carrier)
 
+    labels, n_labels = measure.label((~mask).astype(int), return_num=True)
+    absolute_phase_labels = []
     if 'absolute_phase' in calibration:
-        x_a, y_a, absolute_phase = calibration['absolute_phase']
+        absolute_phases = np.array(calibration["absolute_phase"])
+        if len(absolute_phases.shape) == 1:
+            absolute_phases = np.expand_dims(absolute_phases, 0)
         if isMasked:
-            x_a, y_a = roi.apply_to_points([[x_a, y_a]])[0]
-            if phase.mask[y_a, x_a]:
-                raise ValueError('absolute_phase point must not be masked by segmentation')
-        phase += absolute_phase - carrier[y_a, x_a] - phase[y_a, x_a]
-    else:
-        # setting all labeled areas to have minimum close to phase = 0
-        labels, n_labels = measure.label((~mask).astype(int), return_num=True)
-        sign = int(np.sign(calibration['theta'])) if 'theta' in calibration else 1
-        for j in range(1, n_labels + 1):
-            valid = labels == j
-            points = np.sort(phase[valid].flatten())
-            phase[valid] -= points[int(sign * round(_min_percentile * points.size))]
-            # phase[valid] -= np.min(phase[valid])
+            absolute_phases[:, :2] = roi.apply_to_points(absolute_phases[:, :2])
 
+        for x_a, y_a, absolute_phase in absolute_phases:
+            x_a = int(x_a)
+            y_a = int(y_a)
+            if mask[y_a, x_a]:
+                # value is masked in this array
+                continue
+            absolute_phase_labels.append(labels[y_a, x_a])
+            absolute_phase_area = labels == absolute_phase_labels
+            phase[absolute_phase_area] += absolute_phase - carrier[y_a, x_a] - phase[y_a, x_a]
 
-    xscale = yscale = dscale = 1.
-    if 'theta' in calibration:
-        xscale *= 1 / np.cos(calibration['theta'])
-        dscale *= phase_to_threeD_const(calibration['T'], calibration['theta'])
-    if 'scale' in calibration:
-        xscale /= calibration['scale']
-        yscale /= calibration['scale']
-        dscale /= calibration['scale']
-    if 'phi' in calibration:  ## Phi stuff is not really tested 
-        dscale *= 1 / np.cos(np.abs(calibration['phi'] - calibration['gamma']))
+    # setting all labeled areas without absolute_phase label to have minimum close to phase = 0
+    # sign = int(np.sign(calibration['theta'])) if 'theta' in calibration else 1
+    for j in range(1, n_labels + 1):
+        if j in absolute_phase_labels:
+            continue
+        area = labels == j
+        # points = np.sort(phase[area].flatten())
+        # phase[area] -= points[int(sign * round(_min_percentile * points.size))]
+        phase[area] -= np.min(phase[area])
+
+    xscale, yscale, dscale = get_scales(calibration, signal.shape)
     threeD = phase * dscale
 
     Y, X = np.mgrid[:shape[0], :shape[1]]
@@ -88,9 +108,11 @@ def fp23d(signal, calibration, negative_theta=False):
 
     if 'phi' in calibration:
         phi = calibration['phi']
+        sin_phi = np.sin(phi)
+        cos_phi = np.cos(phi)
         X_copy = X.copy()
-        X = np.cos(phi) * X_copy + np.sin(phi) * Y
-        Y = -np.sin(phi) * X_copy + np.cos(phi) * Y
+        X =  cos_phi * X_copy + sin_phi * Y
+        Y = -sin_phi * X_copy + cos_phi * Y
 
     # Print extent of bounding box for debug
     # print(np.max(X) - np.min(X), np.max(Y) - np.min(Y), np.max(threeD) - np.min(threeD))
@@ -99,9 +121,6 @@ def fp23d(signal, calibration, negative_theta=False):
         mask = threeD.mask
         grid3d = np.ma.stack((np.ma.array(X, mask=mask), np.ma.array(Y, mask=mask), threeD))
         grid3d = roi.unapply(grid3d)
-        # X = roi.unapply(X)
-        # Y = roi.unapply(Y)
-        # threeD = roi.unapply(threeD)
     else:
         grid3d =  np.stack((X, Y, threeD))
     return grid3d
