@@ -162,6 +162,23 @@ class Roi:
                    max(roi1[4], roi2[4]),
                    max(roi1[5], roi2[5])]
         self.roi = roi
+    
+    def enlarge(self, diff):
+        """ make roi diff larger in all directions """
+        self.roi[0] -= diff
+        self.roi[1] += diff
+        self.roi[2] -= diff
+        self.roi[3] += diff
+
+        # Avoid negative values
+        self.roi[0] = max(0, self.roi[0])
+        self.roi[2] = max(0, self.roi[2])
+
+        if len(self.roi) == 6:
+            # with original image shape too large numbers are also avoided
+            self.roi[1] = min(self.roi[4], self.roi[1])
+            self.roi[3] = min(self.roi[5], self.roi[3])
+
         
     def empty(self):
         return self.roi is None
@@ -177,6 +194,10 @@ class Roi:
         w = self.roi[3] - self.roi[2]
         return (h, w)
 
+    def as_pyplot_imshow_extent(self):
+        roi = self.roi
+        return [roi[2], roi[3], roi[1], roi[0]]
+
     def copy(self):
         if self.empty():
             return Roi()
